@@ -27,7 +27,7 @@
 from neoman.ykneomgr import *
 from ctypes import POINTER, byref, c_size_t, create_string_buffer
 from neoman.device import BaseDevice
-from neoman.exc import YkNeoMgrError
+from neoman.exc import YkNeoMgrError, ModeSwitchError
 import os
 
 
@@ -38,8 +38,13 @@ if ykneomgr_global_init(1 if 'NEOMAN_DEBUG' in os.environ else 0) != 0:
 libversion = ykneomgr_check_version(None)
 
 
+U2F_SELECT_1 = '00a4040008a0000006472f0001'.decode('hex')
+U2F_SELECT_2 = '00a4040007a0000005271002'.decode('hex')
+
+
 class CCIDDevice(BaseDevice):
     device_type = 'CCID'
+    version = (0, 0, 0)
 
     def __init__(self, dev, dev_str=None):
         self._dev = dev
@@ -48,19 +53,28 @@ class CCIDDevice(BaseDevice):
         self._locked = True
         self._serial = ykneomgr_get_serialno(dev) or None
         self._mode = ykneomgr_get_mode(dev)
-        self._version = (
-            ykneomgr_get_version_major(dev),
-            ykneomgr_get_version_minor(dev),
-            ykneomgr_get_version_build(dev)
-        )
+        # self._version = (
+        #     ykneomgr_get_version_major(dev),
+        #     ykneomgr_get_version_minor(dev),
+        #     ykneomgr_get_version_build(dev)
+        # )
+        self._supports_u2f = self._has_u2f_applet()
         self._apps = None
         self._broken = False
 
+    def _has_u2f_applet(self):
+        return '\x90\x00' in [self.send_apdu(U2F_SELECT_1)[-2:],
+                              self.send_apdu(U2F_SELECT_2)[-2:]]
+
     def check(self, status):
         if status != 0:
             self._broken = True
             raise YkNeoMgrError(status)
 
+    @property
+    def allowed_modes(self):
+        return (True, True, self._supports_u2f)
+
     @property
     def key(self):
         return self._key
@@ -81,10 +95,6 @@ class CCIDDevice(BaseDevice):
     def serial(self):
         return self._serial
 
-    @property
-    def version(self):
-        return self._version
-
     def unlock(self):
         self._locked = True
         if not self._key:
@@ -93,7 +103,8 @@ class CCIDDevice(BaseDevice):
         self._locked = False
 
     def set_mode(self, mode):
-        self.check(ykneomgr_modeswitch(self._dev, mode))
+        if ykneomgr_modeswitch(self._dev, mode) != 0:
+            raise ModeSwitchError()
         self._mode = mode
 
     def send_apdu(self, apdu):
