@@ -43,6 +43,9 @@ ok = "OK"
 cancel = "Cancel"
 wait = "Please wait..."
 welcome = "Welcome"
+welcome_desc = "No device found.\n\n\n" \
+    "Please insert a YubiKey to continue..."
+
 note_1 = "NOTE: %s"
 overview = "Overview"
 otp = "OTP"
@@ -60,15 +63,19 @@ change_name = "Change name"
 change_name_desc = "Change the name of the device."
 manage_keys = "Manage transport keys"
 key_required = "Transport key required"
-key_required_desc = "Managing apps on this YubiKey NEO requires a password"
+key_required_desc = "Managing apps on this YubiKey requires a password"
 change_mode = "Change connection mode"
 change_mode_1 = "Change connection mode [%s]"
-change_mode_desc = ("Set the connection mode used by your YubiKey NEO.\nFor "
+change_mode_desc = ("Set the connection mode used by your YubiKey.\nFor "
                     "this setting to take effect, you will need to unplug, "
                     "and re-attach your YubiKey.")
-remove_device = "\nRemove your YubiKey NEO now.\n"
-mode_note = ("To be able to list/manage apps, your YubiKey NEO must have CCID "
+remove_device = "\nRemove your YubiKey now.\n"
+mode_note = ("To be able to list/manage apps, your YubiKey must have CCID "
              "enabled.")
+mode_error = "Error setting mode"
+mode_error_desc = "Failed setting the mode. If you have an Access Code " \
+    "protecting either of the YubiKey slots, you will need to disable this " \
+    "before you can change the mode."
 name = "Name"
 name_1 = "Name: %s"
 serial_1 = "Serial: %s"
